const
    generators  = require('yeoman-generator'),
    walk        = require('walk'),
    path        = require('path'),
    yosay       = require('yosay'),
    chalk       = require('chalk');

module.exports = generators.Base.extend({
    initializing: {
        acceptArgs: function(){
            this.argument('appname', {type:String, required:false});
        }
    },
    prompting: {
        sayYo: function() {
            this.log(
                yosay(chalk.red('Hello!')+
                ' Welcome to my generator for amazing '+
                chalk.cyan('angular plugin!'))
            );
        },
        
        askAppName: function() {
            var done = this.async();
            var that = this;
            
            this.prompt({
                type: 'input',
                name: 'appname',
                message: 'Your project name',
                default: this.appname
            }, function(answers) {
                that.appname = answers.appname;
                done();
                
            });
        },
        
        askDescription: function() {
            var done = this.async();
            var that = this;
            
            this.prompt({
                type: 'input',
                name: 'description',
                message: 'Your description',
                default: 'Angular plugin for '+this.appname
            }, function(answers) {
                that.description = answers.description;
                done();
            });
        },
        
        askMainFile: function() {
            var done = this.async();
            var that = this;
            
            this.prompt({
                type: 'input',
                name: 'mainfile',
                message: 'Your main file name[include .js]',
                default: this.appname+'.js'
            }, function(answers){
                that.mainfile = answers.mainfile;
                done();
            })
        },
        
        askModuleName: function() {
            var done = this.async();
            var that = this;
            
            function camelCase(str) {
                return str.split('-').reduce(function(pre, cur){
                    pre += cur.charAt(0).toUpperCase()+cur.slice(1);
                    return pre;
                });
            }
            
            
            this.prompt({
                type:'input',
                name: 'modulename',
                message: 'Your module name',
                default: camelCase(this.appname)
            }, function(answers){
                that.modulename = answers.modulename;
                done();
            })
            
        }
    },
    
    writing: {
        copyTemplates: function() {
            this.log.ok(chalk.magenta.underline.bold('Start copying templates into your directory.'));
            
            
            var rootPath = this.templatePath('root');
            var walker = walk.walk(rootPath);
            var that = this;
            var done = this.async();
            
            walker.on('file', function(root, fileStats, next){
                var relative = root.slice(rootPath.length+1);
                var fileName = fileStats.name;
                var newName = fileName.indexOf('_') === 0 ? fileName.slice(1) : fileName;
                
                var fPath = path.join(root, fileName);
                var dPath = path.join(that.destinationPath(), relative, newName);
                
                
                if( fileName.indexOf('_') === 0 ) {
                    that.fs.copyTpl(fPath, dPath, that);
                }
                else {
                    that.fs.copy(fPath, dPath);
                }
                next();
            });
        
            walker.on('end', done);
        }
    },
    
    install: {
        installDependency: function() {
            this.installDependencies();
            this.log('If don\'t want to run now, press '+chalk.yellow('ctrl + c')+'.\n');
        }
    }
});